# runSimulationSetScript.R

################################################################################################

# magnitude of simulated gradient change at event from background, relative to total gradient length
  # abs(peakGradientValue - bgGradientValue) / fullGradientRange
# NOTE cannot define a peak that is further than the max DCA-1 of original data
eventGradientChange_paramValues <- get_eventGradientChange_paramValues(
    bgGradientValue = bgGradientValue
    )

# bioturbation depth ratio 
  # the unit-less ratio of the bioturbation depth (cm)  
  # to the sample width (cm) in the simulation.
# IN OTHER WORDS BIOTURBATION DEPTH /OVER/ SAMPLE WIDTH 
bioturbDepthRatio <- 10/3

# fixed foram accumulation per year (or horizon)
foramsPerYear <- 400

# samples represent multiple years / horizons -- lump them together
# subsample a lumped assemblage to the 'picked' sample size
nSpecimensPicked <- 300
  
## Simulation Acceptance/Rejection Checks
# Should checks of the simulated data be run before doing ordinations or other analyses?
runChecks <- TRUE
# these checks mainly don't apply when picked samples sizes are fixed

#The following parameters adjust checks that *shouldn't* have any effect 
  # when using fixed sample size (300) but are included here anyway for completeness.
# minimum number of picked specimens in practice is 20
minPickedSampleSize <- 20
# samples larger than maxPickedSampleSize get split
    # in practice this is around 600
maxPickedSampleSize <- 600
# any sample where a single taxon accounts for more than maxDominance
  # will be considered as 'over dominated'
maxDominance <- 0.9

# To calculate the necessary parameters, we need to set *two* (and only two) of *three* arbitrary 'nuisance' parameters: 
  # **sample width** (in cm),
  # the **duration** that an event 'plateaus' at some peak value (in years) 
  # or **sedimentation rate** (in cm per year). 
sampleWidth <- 3 # cm
eventDuration <- 20 #years
sedRatePerYear <- NULL #cm/year

## Load Empirical Gradient Data
# Loading the Empirical Ecological Dataset For Generating Simulated Species Distributions via KDEs

# full Sharon et al. dataset
if(origDatasetSelection == "SharonEtAl"){
    origAbundDataTable <- read.table(
        "data/foram_abundances_forSimulations.txt",
        header = TRUE, row.names = 1)
    # remove first four columns of data that aren't abundances
    origAbundData <- origAbundDataTable[, -(1:4)] 
    # get DCA1
    gradientOrigDCA <- origAbundDataTable$DCA1
    }

if(origDatasetSelection == "2016"){ 
    origAbundDataTable <- read.csv(
        "data/SimulationSubsampleTest_option3.csv"
        , row.names = 1)
    # get the original abundance data
    origAbundData <- origAbundDataTable[,-(1:5)]
    origAbundData <- origAbundData[-which(apply(origAbundData,2,sum)==0)]
    # get DCA values from relative abundance
    relOrigAbundData <- t(apply(origAbundData, 1, function(x) x/sum(x)))
    gradientOrigDCA <- vegan::scores(vegan::decorana(relOrigAbundData))[,"DCA1"]  #$rproj[,"DCA1"]
    }

##################################################

# Calculate full gradient range
fullGradientRange <- c(min(gradientOrigDCA), max(gradientOrigDCA))

# DCA calculation modes
singularDCA <- TRUE
inclusiveDCA <- TRUE
rawDCA <- FALSE

###############################################################################

outputDir<- paste0(outputDir_main, set_name, "/")
if (!dir.exists(outputDir)) {
    dir.create(path = outputDir)
    }

set_name <- paste0(set_name, "_", as.character(format(Sys.Date(),"%m-%d-%y")))
eventDataOut_filename <- paste0(
    outputDir_main,
    "saved_postSimAnalysis_", 
    set_name, 
    ".Rdata")

# check that necessary packages are loaded
packageVersion("vegan")

# get all param combinations
paramCombos <- expand.grid(eventGradientChange_paramValues, eventSampleWidth_paramValues)
# nrow(paramCombos)
colnames(paramCombos) <- c("eventGradientChange_param", "eventSampleWidth_param")

###########################################################
# Get Ecological Occurrence Simulation Functions

# Fit rescaled kernal density estimates for each species along axis DCA-1.
kdeRescaled <- getSpeciesSpecificRescaledKDE(
    gradientOrigDCA = gradientOrigDCA, 
    origAbundData = origAbundData,
    nBreaksGradientHist = nBreaksGradientHist)

# Make a function for sampling species presence 
  # at a point along the DCA gradient 
  # based solely on presence and absence
sampleSpeciesGradient <- getProbOccViaPresAbs(
    gradientOrigDCA = gradientOrigDCA, 
    origAbundData = origAbundData)

####################################################################

# estimate envelope on background gradient values 
backgroundUpperBound <- simulateGradientQuantile(
    nSamplesSim = 500,
    gradientValue = bgGradientValue,
    origAbundData = origAbundData,
    kdeRescaled = kdeRescaled,
    sampleSpeciesGradient = sampleSpeciesGradient,
    foramsPerYear = foramsPerYear,
    nSpecimensPicked = nSpecimensPicked
    )    

# get envelopes on event magnitudes
eventMagnitudesAbsolute <- (eventGradientChange_paramValues * diff(fullGradientRange)) + bgGradientValue
#
simMagEnvelopes <- sapply(eventMagnitudesAbsolute,
    function(x) simulateGradientQuantile(
        gradientValue = x, 
        quantileProbs = c(0.025, 0.975),  # two sided 95% quantiles
        nSamplesSim = 500,
        origAbundData = origAbundData,
        kdeRescaled = kdeRescaled,
        sampleSpeciesGradient = sampleSpeciesGradient,
        foramsPerYear = foramsPerYear,
        nSpecimensPicked = nSpecimensPicked
        )
    )
    
simMagEnvelopes <- t(simMagEnvelopes)
simMagEnvelopes <- data.frame(
    eventChangeScale = eventGradientChange_paramValues,
    eventMagnitudesAbsolute = eventMagnitudesAbsolute, 
    lowerBound = simMagEnvelopes[,1], 
    upperBound = simMagEnvelopes[,2]
    )

#####################################################################
# make a data frame for storing simulation summary statistics
simSummaryTable <- paramCombos
# add additional variables
simSummaryTable$nEvents <- nEvents
simSummaryTable$backgroundMedian <- NA
simSummaryTable$backgroundUpperBound <- NA
simSummaryTable$nEventsDetected <- NA
#simSummaryTable$nEventsWithinExpectedMagnitude <- NA
simSummaryTable$nDetectedEventsWithinExpectedMagnitude <- NA
simSummaryTable$medianMaxDCA_acrossEvents <- NA

simSetList <- list(
    simSetDCAvalues = list(),
    meanDCA = list(),
    medianDCA = list(),
    maxDCA = list()
    )

message(paste0("Running first of ", 
    nrow(paramCombos), " simulations in this set..."))

# big for() loop
for(i in 1:nrow(paramCombos)){
  
    eventChangeScale <- paramCombos$eventGradientChange_param[i]
    eventSampleWidthRatio <- paramCombos$eventSampleWidth_param[i]
  
    simulateCoreRecord_out <- simulateCoreRecord(
            kdeRescaled = kdeRescaled,
            sampleSpeciesGradient = sampleSpeciesGradient,
            origAbundData = origAbundData,
            eventChangeScale = eventChangeScale,
            bgGradientValue = bgGradientValue,
            fullGradientRange = fullGradientRange,
            eventSampleWidthRatio = eventSampleWidthRatio,
            sampleWidth = sampleWidth,
            eventDuration = eventDuration,
            sedRatePerYear = sedRatePerYear,
            samplingCompleteness = samplingCompleteness,
            transitionDurationRatio = transitionDurationRatio,
            bioturbDepthRatio = bioturbDepthRatio,
            bioturbIntensity = bioturbIntensity, 
            nEvents = nEvents,
            foramsPerYear = foramsPerYear,
            nSpecimensPicked = nSpecimensPicked,
            singularDCA = singularDCA,
            inclusiveDCA = inclusiveDCA,
            rawDCA = rawDCA,
            includeInitialBackgroundPhase = FALSE,
            plot = plot
            )
    
    simSummaryOut <- summarizeSimulationDCA(
        sampleData = simulateCoreRecord_out$sampleProperties,
        nEvents = nEvents,
        backgroundUpperBound = backgroundUpperBound,
        eventMagnitudesAbsolute = simMagEnvelopes$eventMagnitudesAbsolute[simMagEnvelopes$eventChangeScale == eventChangeScale],
        eventLowerBound = simMagEnvelopes$lowerBound[simMagEnvelopes$eventChangeScale == eventChangeScale],
        eventUpperBound = simMagEnvelopes$upperBound[simMagEnvelopes$eventChangeScale == eventChangeScale]
        )
    
    simSummaryTable <- addToSimSumTable(
        simSummaryTable = simSummaryTable,
        simSummaryOut = simSummaryOut)
    
    ##########################

    simSetList <- addToSimSetList(
        simSetList = simSetList,
        simSummaryOut = simSummaryOut)
            
    ###################################################################
    # plot original simulated gradient versus estimated DCA values
    if(plot){
        plotCoreSimDCA(
            simTimeVar = simulateCoreRecord_out$simTimeVar, 
            scoreDCA1 = simulateCoreRecord_out$ecology$scoreDCA1,
            sampleAge = simulateCoreRecord_out$sampleProperties$sampleMidAge
            )
        }

    #############################
    # save workspace
    save(
        list = ls(),
        file = paste0(
            outputDir,"savedEnv_run", sprintf("%04d", i),
            "_gradientScale", round(paramCombos$eventGradientChange_param[i], 3), 
            "_widthRatio", round(paramCombos$eventSampleWidth_param[i], 3),
            "_", format(Sys.Date(),"%m-%d-%y"), 
            ".Rdata"
            )
        )
    }

# get the pooled median for event DCA values
simSummaryTable$medianPooledDCA <-  sapply(simSetList$simSetDCAvalues, 
    function(x) median(unlist(x), na.rm = TRUE))
# get the median of the maximum DCA observed for all events
simSummaryTable$medianMaxDCA_acrossEvents <- sapply(simSetList$maxDCA, 
    function(x) median(unlist(x), na.rm = TRUE))

simSetSummaryList <- summarizeSimulationSet(
    simSummaryTable = simSummaryTable,
    eventGradientChange_paramValues = eventGradientChange_paramValues, 
    eventSampleWidth_paramValues = eventSampleWidth_paramValues
    )

save(list = ls(), file = eventDataOut_filename)


